import { HttpException, HttpStatus, Injectable, Logger } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { MailerService } from '../mailer/mailer.service';
import { AppointmentDto } from './dto/appointment.dto';
import { IAppointment } from './interfaces/appointment.interface';
import { Appointment, AppointmentDocument } from './appointment.schema';
import * as ejs from 'ejs';
import { StructuresService } from '../structures/services/structures.service';
import { UsersService } from '../users/services/users.service';
import { StructureDocument } from '../structures/schemas/structure.schema';
import { IUser } from '../users/interfaces/user.interface';

@Injectable()
export class AppointmentService {
  private readonly logger = new Logger(AppointmentService.name);
  private appointment: AppointmentDto;
  private structureRDV: StructureDocument;
  private socialWorkerUser: IUser;

  constructor(
    @InjectModel(Appointment.name) private AppointmentModel: Model<AppointmentDocument>,
    private usersService: UsersService,
    private structureService: StructuresService,
    private readonly mailerService: MailerService
  ) {}

  public async create(newAppointment: AppointmentDto): Promise<IAppointment> {
    this.appointment = newAppointment;

    // Get structure name
    this.structureRDV = await this.structureService.findOne(newAppointment.structureRDV);
    if (!this.structureRDV) {
      throw new HttpException(`Unknown structure : ${newAppointment.structureRDV}`, HttpStatus.NOT_FOUND);
    }

    // Get social worker user
    this.socialWorkerUser = await this.usersService.findById(newAppointment.socialWorkerId);
    if (!this.socialWorkerUser) {
      throw new HttpException(`Unknown user : ${newAppointment.socialWorkerId}`, HttpStatus.NOT_FOUND);
    }
    this.socialWorkerUser.surname = this.socialWorkerUser.surname.toUpperCase();

    try {
      newAppointment.surname = newAppointment.surname.toUpperCase();
      await this.sendMail();
      return await this.AppointmentModel.create(newAppointment);
    } catch (e) {
      this.logger.error(`Creation Error : ${e.message}`);
      throw new HttpException(`Creation Error  : ${e.message}`, HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }

  public async findAll(): Promise<IAppointment[]> {
    return this.AppointmentModel.find().exec();
  }

  private async sendMail(): Promise<null> {
    this.logger.debug('AppointmentService sendMail');

    const config = this.mailerService.config;
    const ejsPath = this.mailerService.getTemplateLocation(config.templates.appointment.ejs);
    const jsonConfig = this.mailerService.loadJsonConfig(config.templates.appointment.json);

    const html = await ejs.renderFile(ejsPath, {
      config,
      appointment: this.appointment,
      structureRDV: this.structureRDV,
      socialWorkerUser: this.socialWorkerUser,
    });
    this.mailerService.send(this.socialWorkerUser.email, jsonConfig.subject, html);
    return;
  }
}
