import { HttpService } from '@nestjs/axios';
import { HttpException, HttpStatus, Injectable, Logger } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model } from 'mongoose';
import { lastValueFrom } from 'rxjs';
import slugify from 'slugify';
import { Categories, CategoriesDocument } from '../schemas/categories.schema';
import { Module } from '../schemas/module.class';

@Injectable()
export class CategoriesService {
  private readonly logger = new Logger(CategoriesService.name);
  constructor(
    @InjectModel(Categories.name) private categoriesModel: Model<CategoriesDocument>,
    private httpService: HttpService
  ) {}

  public async findAll(): Promise<Categories[]> {
    this.logger.debug(`findAll`);
    return this.categoriesModel.find().exec();
  }

  public findOneComplete(categoryId: string): Promise<Categories> {
    this.logger.debug(`findOneComplete`);
    return this.categoriesModel.findOne({ id: categoryId }).exec();
  }

  public async findCtmByInseeCode(inseeCode: string): Promise<Module> {
    const ctm = await this.findOneComplete('ctm');
    if (!ctm) {
      this.logger.error(`CTM territories definition not found. For local dev, run init-ctm-territories.js script.`);
      throw new HttpException(
        `CTM territories definition not found in database. For local dev, run init-ctm-territories.js script.`,
        HttpStatus.INTERNAL_SERVER_ERROR
      );
    }
    return ctm.modules.find((ctm) => ctm.communes.includes(inseeCode));
  }

  /**
   * Clear 'categories.ctm' and fill it with data from Data Grand Lyon
   */
  public async updateCTM(): Promise<Categories> {
    this.logger.debug('updateCTM');
    const dataCTMs = await lastValueFrom(
      this.httpService.get(
        `https://download.data.grandlyon.com/ws/grandlyon/ter_territoire.direction_territoriale/all.json?maxfeatures=-1`
      )
    ).then(async (res) => res.data.values);
    dataCTMs.sort((a, b) => a.nom.localeCompare(b.nom));

    const newDocCTM: Categories = {
      modules: [],
      name: 'Territoires métropolitains',
      theme: 'Territoires métropolitains',
      id: 'ctm',
    };

    for (const ctm of dataCTMs) {
      newDocCTM.modules.push({
        id: slugify(ctm.nom, { lower: true, strict: true, locale: 'fr' }),
        name: ctm.nom,
        communes: ctm.communes,
      });
    }

    await this.categoriesModel.updateOne({ id: 'ctm' }, [{ $set: newDocCTM }], { upsert: true }).exec();
    return newDocCTM;
  }
}
