import { Db } from 'mongodb';
import { StructureDocument } from '../../structures/schemas/structure.schema';
import { getDb } from '../migrations-utils/db';

export const up = async () => {
  const db: Db = await getDb();

  const cursor = db.collection('structures').find({});

  // Create categories fields
  let document;
  while ((document = await cursor.next())) {
    const newDoc: StructureDocument = upgradeStructure(document);
    await db.collection('structures').updateOne({ _id: document._id }, [{ $set: newDoc }]);
  }
  console.log(`Update done : new structrure repository ref`);
  try {
    await db.dropCollection('categoriesaccompagnements');
    await db.dropCollection('categoriesformations');
    await db.dropCollection('categoriesothers');
  } catch (e) {
    if (e.code === 26) {
      console.warn('Already deleted collections');
    }
  }
  console.log(`Remove old referential done`);
};

export const down = async () => {
  const db: Db = await getDb();
  const cursor = db.collection('structures').find({});
  let document;
  while ((document = await cursor.next())) {
    const newDoc: StructureDocument = downgradeStructure(document);
    await db.collection('structures').updateOne({ _id: document._id }, [{ $set: newDoc }]);
  }
  console.log(`Downgrade done : new structrure repository removed`);
};

const downgradeStructure = (doc: StructureDocument): StructureDocument => {
  doc.categories = {};
  delete doc.categories;
  return doc;
};

const upgradeStructure = (doc: any): StructureDocument => {
  doc.categories = {};
  doc = mapCategories(doc);
  doc = mapOnlineProcedures(doc);
  doc = mapEquipments(doc);
  doc = mapWorkInsertion(doc);
  doc = mapDigitalCultureSecurity(doc);
  doc = mapbaseComp(doc);
  doc = mapParentingHelp(doc);
  doc = mapPublics(doc);
  delete doc.nbNumericTerminal;
  delete doc.nbTablets;
  delete doc.publicsAccompaniment;
  delete doc.proceduresAccompaniment;
  delete doc.accessModality;
  delete doc.labelsQualifications;
  delete doc.publics;
  delete doc.equipmentsAndServices;
  delete doc.baseSkills;
  delete doc.accessRight;
  delete doc.parentingHelp;
  delete doc.socialAndProfessional;
  delete doc.digitalCultureSecurity;
  delete doc.equipmentsServices;
  return doc;
};

const mapCategories = (doc: any): StructureDocument => {
  doc.categories.accessModality = doc.accessModality || [];
  doc.categories.labelsQualifications = doc.labelsQualifications || [];
  return doc;
};

const mapEquipments = (doc: any): StructureDocument => {
  const oldMaterial = doc.equipmentsAndServices;
  const selfServiceMaterial = oldMaterial
    ?.map((el) => {
      switch (el) {
        case 'ordinateurs':
          return 'computer';
        case 'imprimantes':
          return 'printer';
        case 'wifiEnAccesLibre':
          return 'wifiEnAccesLibre';
        case 'scanners':
          return 'scanner';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  const equipmentsServices = oldMaterial
    ?.map((el) => {
      switch (el) {
        case 'donDeMateriels':
          return 'donDeMateriels';
        case 'reconditionnementsDeMateriel':
          return 'reconditionnementsDeMateriel';
        case 'accesLivresInformatiques':
          return 'accesLivresInformatiques';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  doc.categories.selfServiceMaterial = [...new Set(selfServiceMaterial || [])];
  doc.categories.equipmentsServices = [...new Set(equipmentsServices || [])];
  return doc;
};

const mapOnlineProcedures = (doc: any): StructureDocument => {
  const oldModality = doc.accessRight;
  const oldHelping = doc.proceduresAccompaniment;
  const newModality = oldModality?.map((el) => {
    switch (el) {
      case '84':
        return 'work';
      case '85':
        return 'taxes';
      case '86':
        return 'caf';
      case '87':
        return 'retirement';
      case '88':
        return 'needs';
      case '89':
        return 'caf';
      case '93':
        return 'health';
      case '95':
        return 'franceConnect';
      default:
        return '';
    }
  });
  const newHelping = oldHelping?.map((el) => {
    switch (el) {
      case 'accompagnantCaf':
        return 'caf';
      case 'carsat':
        return 'retirement';
      case 'cpam':
        return 'cpam';
      case 'impots':
        return 'taxes';
      case 'logement':
        return 'housing';
      case 'poleEmploi':
        return 'franceConnect';
      case 'autres':
        return 'autres';
      default:
        return '';
    }
  });
  const onlineProcedures = [...new Set([...(newModality || []), ...(newHelping || [])].filter((el) => el !== ''))];
  doc.categories.onlineProcedures = onlineProcedures;
  return doc;
};

const mapWorkInsertion = (doc: any): StructureDocument => {
  const oldSocialAndPro = doc.socialAndProfessional;
  const baseSkills = oldSocialAndPro
    ?.map((el) => {
      switch (el) {
        case '6':
          return 'work';
        case '20':
          return 'work';
        case '66':
          return 'officeAutomation';
        case '67':
          return 'officeAutomation';
        case '68':
          return 'officeAutomation';
        case '69':
          return 'officeAutomation';
        case '124':
          return 'work';
        case '125':
          return 'work';
        case '127':
          return 'work';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  doc.categories.baseSkills = [...new Set(baseSkills || [])];
  return doc;
};
const mapDigitalCultureSecurity = (doc: any): StructureDocument => {
  const oldData = doc.digitalCultureSecurity;
  const baseSkills = oldData
    ?.map((el) => {
      switch (el) {
        case '5':
          return 'communication';
        case '34':
          return 'internet';
        case '51':
          return 'data';
        case '52':
          return 'data';
        case '54':
          return 'data';
        case '65':
          return 'communication';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  const advancedSkills = oldData
    ?.map((el) => {
      switch (el) {
        case '2':
          return 'cyberAndConf';
        case '9':
          return 'numericCreate';
        case '39':
          return 'fakeNews';
        case '96':
          return 'cyberAndConf';
        case '97':
          return 'cyberAndConf';
        case '98':
          return 'cyberAndConf';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  doc.categories.baseSkills = [...new Set([...(baseSkills || []), ...doc.categories.baseSkills])];
  doc.categories.advancedSkills = [...new Set(advancedSkills)];
  return doc;
};

const mapbaseComp = (doc: any): StructureDocument => {
  const oldData = doc.baseSkills;
  const baseSkills = oldData
    ?.map((el) => {
      switch (el) {
        case '260':
          return 'computer';
        case '11':
          return 'internet';
        case '38':
          return 'smartphone';
        case '48':
          return 'mail';
        case '74':
          return 'smartphone';
        case '77':
          return 'smartphone';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  const advancedSkills = oldData
    ?.map((el) => {
      switch (el) {
        case '1':
          return 'cyberAndConf';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  doc.categories.baseSkills = [...new Set([...(baseSkills || []), ...doc.categories.baseSkills])];
  doc.categories.advancedSkills = [...new Set([...(advancedSkills || []), ...doc.categories.advancedSkills])];
  return doc;
};
const mapParentingHelp = (doc: any): StructureDocument => {
  const oldData = doc.parentingHelp;
  const baseSkills = oldData
    ?.map((el) => {
      switch (el) {
        case '82':
          return 'scolarity';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  const advancedSkills = oldData
    ?.map((el) => {
      switch (el) {
        case '3':
          return 'parenting';
        case '22':
          return 'videoGames';
        default:
          return '';
      }
    })
    .filter((el) => el !== '');
  doc.categories.baseSkills = [...new Set([...(baseSkills || []), ...doc.categories.baseSkills])];
  doc.categories.advancedSkills = [...new Set([...(advancedSkills || []), ...doc.categories.advancedSkills])];
  return doc;
};
const mapPublics = (doc: any): StructureDocument => {
  const age = [];
  const languageAndIlliteracy = [];
  const handicaps = [];
  const publicOthers = [];
  const solidarityMaterial = [];
  doc.publics?.forEach((el) => {
    switch (el) {
      case 'toutPublic':
        age.push('family');
        age.push('young');
        age.push('adults');
        age.push('old');
        break;
      case 'moinsDe16Ans':
        age.push('family');
        break;
      case 'jeunes1625Ans':
        age.push('young');
        break;
      case 'adultes':
        age.push('adults');
        break;
      case 'seniorsPlusDe65Ans':
        age.push('old');
        break;
      case 'uniquementFemmes':
        publicOthers.push('uniquementFemmes');
        break;
      default:
        break;
    }
  });

  doc.categories.age = [...new Set([...age])];
  doc.publicsAccompaniment?.forEach((el) => {
    switch (el) {
      case "Personnes en situation d'illetrisme":
        languageAndIlliteracy.push('illettrisme');
        break;
      case 'Langue étrangère (anglais)':
        languageAndIlliteracy.push('english');
        break;
      case 'Langues étrangères (autres)':
        languageAndIlliteracy.push('foreignLanguage');
        break;
      case 'Surdité':
        handicaps.push('deafness');
        break;
      case 'Déficience visuelle':
        handicaps.push('blindness');
        break;
      case 'Handicap moteur':
        handicaps.push('physicalDisability');
        break;
      default:
        break;
    }
  });

  doc.categories.age = age;
  doc.categories.languageAndIlliteracy = languageAndIlliteracy;
  doc.categories.handicaps = handicaps;
  doc.categories.publicOthers = publicOthers;
  doc.categories.solidarityMaterial = solidarityMaterial;
  return doc;
};
