import { HttpModule } from '@nestjs/axios';
import { HttpStatus } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import { INewsletterSubscription } from './interface/newsletter-subscription.interface';
import { NewsletterSubscription } from './newsletter-subscription.schema';
import { NewsletterService } from './newsletter.service';
import { UsersService } from '../users/services/users.service';
// eslint-disable-next-line @typescript-eslint/no-var-requires
const mailchimp = require('@mailchimp/mailchimp_marketing');
jest.mock('@mailchimp/mailchimp_marketing');
describe('NewsletterService', () => {
  const OLD_ENV = process.env;
  let newsletterService: NewsletterService;

  const mockNewsletterModel = {
    create: jest.fn(),
    deleteOne: jest.fn(),
    countDocuments: jest.fn(),
    findOne: jest.fn(),
    exec: jest.fn(),
    find: jest.fn(),
  };

  const mockUsersService = {
    findOne: jest.fn(),
  };

  beforeEach(async () => {
    jest.resetModules(); // Most important - it clears the cache
    const module = await Test.createTestingModule({
      imports: [HttpModule],
      providers: [
        NewsletterService,
        {
          provide: getModelToken(NewsletterSubscription.name),
          useValue: mockNewsletterModel,
        },
        {
          provide: UsersService,
          useValue: mockUsersService,
        },
      ],
    }).compile();

    newsletterService = module.get<NewsletterService>(NewsletterService);

    process.env = { ...OLD_ENV }; // Make a copy
    process.env.MC_LIST_ID = 'abcde';
    process.env.MC_API_KEY = 'k3y';
    process.env.MC_SERVER = 's3rv3r';
    jest.clearAllMocks();
  });

  afterAll(() => {
    process.env = OLD_ENV; // Restore old environment
  });

  it('should be defined', () => {
    expect(newsletterService).toBeDefined();
  });

  describe('newsletterSubscribe', () => {
    it('should add subscription for email test2@test.com even if it exists', async () => {
      const _doc = { _id: 'a1aaaaa1a1', email: 'test2@test.com' } as INewsletterSubscription;
      mailchimp.lists.setListMember.mockResolvedValueOnce({ email_address: 'test2@test.com' });
      mailchimp.lists.updateListMemberTags.mockResolvedValueOnce({});
      jest
        .spyOn(newsletterService, 'findOne')
        .mockImplementationOnce(async (): Promise<INewsletterSubscription | undefined> => _doc);
      mockNewsletterModel.create.mockResolvedValueOnce(_doc);

      const subscription = await newsletterService.newsletterSubscribe('test2@test.com');
      expect(subscription).toEqual(_doc);
    });
    it('should add a subscription for email test2@test.com', async () => {
      const result = { email: 'test2@test.com' } as INewsletterSubscription;
      const _doc = { _id: 'a1aaaaa1a1', email: 'test2@test.com' };
      mailchimp.lists.setListMember.mockResolvedValueOnce({ email_address: 'test2@test.com' });
      mailchimp.lists.updateListMemberTags.mockResolvedValueOnce({});
      jest
        .spyOn(newsletterService, 'findOne')
        .mockImplementationOnce(async (): Promise<INewsletterSubscription | undefined> => undefined)
        .mockImplementationOnce(async (): Promise<INewsletterSubscription | undefined> => result);
      mockNewsletterModel.create.mockResolvedValueOnce(_doc);

      const subscription = await newsletterService.newsletterSubscribe('test2@test.com');
      expect(subscription).toEqual(_doc);
    });
    it('should return error if mailchimp 400 issue', async () => {
      const result = { email: 'test2@test.com' } as INewsletterSubscription;
      const _doc = { _id: 'a1aaaaa1a1', email: 'test2@test.com' };
      mailchimp.lists.setListMember.mockRejectedValueOnce({ status: 400 });
      mailchimp.lists.updateListMemberTags.mockResolvedValueOnce({});
      jest
        .spyOn(newsletterService, 'findOne')
        .mockImplementationOnce(async (): Promise<INewsletterSubscription | undefined> => undefined)
        .mockImplementationOnce(async (): Promise<INewsletterSubscription | undefined> => result);
      mockNewsletterModel.create.mockResolvedValueOnce(_doc);

      try {
        await newsletterService.newsletterSubscribe('test2@test.com');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Subscribe error');
        expect(e.status).toEqual(HttpStatus.BAD_REQUEST);
      }
    });
    it('should return error if mailchimp 500 issue', async () => {
      const result = { email: 'test2@test.com' } as INewsletterSubscription;
      const _doc = { _id: 'a1aaaaa1a1', email: 'test2@test.com' };
      mailchimp.lists.setListMember.mockRejectedValueOnce({ status: 500 });
      mailchimp.lists.updateListMemberTags.mockResolvedValueOnce({});
      jest
        .spyOn(newsletterService, 'findOne')
        .mockImplementationOnce(async (): Promise<INewsletterSubscription | undefined> => undefined)
        .mockImplementationOnce(async (): Promise<INewsletterSubscription | undefined> => result);
      mockNewsletterModel.create.mockResolvedValueOnce(_doc);

      try {
        await newsletterService.newsletterSubscribe('test2@test.com');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Subscribe error');
        expect(e.status).toEqual(HttpStatus.INTERNAL_SERVER_ERROR);
      }
    });
  });
  describe('newsletterUnsubscribe', () => {
    it('should not remove subscription for email test@test.com : does not exist', async () => {
      mailchimp.lists.getListMember.mockRejectedValueOnce({ status: 404 });
      mockNewsletterModel.findOne.mockReturnThis();
      mockNewsletterModel.exec.mockResolvedValueOnce(undefined);
      try {
        await newsletterService.newsletterUnsubscribe('test@test.com');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toEqual('Email not found');
        expect(e.status).toEqual(HttpStatus.NOT_FOUND);
      }
    });
    it('should remove a subscription for email test2@test.com', async () => {
      const _doc = { _id: 'a1aaaaa1a1', email: 'test2@test.com' };
      mailchimp.lists.getListMember.mockResolvedValueOnce({ email_address: 'test2@test.com' });
      mailchimp.lists.setListMember.mockResolvedValueOnce({ email_address: 'test2@test.com' });
      const result = {
        email: 'test2@test.com',
        deleteOne: async () => _doc,
      } as INewsletterSubscription;
      jest
        .spyOn(newsletterService, 'findOne')
        .mockImplementationOnce(async (): Promise<INewsletterSubscription | undefined> => result);

      const subscription = await newsletterService.newsletterUnsubscribe('test2@test.com');
      expect(subscription).toEqual(_doc);
    });
  });

  describe('findOne', () => {
    it('should not find a subscription with email test@test.com', async () => {
      mockNewsletterModel.findOne.mockReturnThis();
      mockNewsletterModel.exec.mockResolvedValueOnce(undefined);
      const findOneEmail = await newsletterService.findOne('test@test.com');
      expect(findOneEmail).toEqual(undefined);
    });
    it('should find a subscription with email test2@test.com', async () => {
      const _doc = { _id: 'a1aaaaa1a1', email: 'test2@test.com' } as INewsletterSubscription;
      mockNewsletterModel.findOne.mockReturnThis();
      mockNewsletterModel.exec.mockResolvedValueOnce(_doc);
      const findOneEmail = await newsletterService.findOne('test2@test.com');
      expect(findOneEmail).toEqual(_doc);
    });
  });
  describe('findAll', () => {
    it('should find all', async () => {
      const _docs = [{ _id: 'a1aaaaa1a1', email: 'test2@test.com' } as INewsletterSubscription];
      mockNewsletterModel.find.mockReturnThis();
      mockNewsletterModel.exec.mockResolvedValueOnce(_docs);
      const findOneEmail = await newsletterService.findAll();
      expect(findOneEmail).toEqual(_docs);
    });
  });
  describe('updateNewsletterSubscription', () => {
    it('should update existing user subscription', async () => {
      mailchimp.lists.getListMembersInfo.mockResolvedValueOnce({ total_items: 10 }).mockResolvedValueOnce({
        members: [
          { email_address: 'a@a.com', status: 'subscribed' },
          { email_address: 'test@test.com', status: 'unsubscribed' },
          { email_address: 'test2@test.com', status: 'unsubscribed' },
        ],
      });
      const result = { email: 'test2@test.com', deleteOne: jest.fn() };
      mockNewsletterModel.findOne.mockReturnThis();
      mockNewsletterModel.exec.mockResolvedValueOnce(result).mockResolvedValueOnce(null);
      await newsletterService.updateNewsletterSubscription();
      expect(mockNewsletterModel.findOne).toHaveBeenCalledTimes(2);
      // Remove subscription is not tested
      // expect(spyerDelete).toBeCalledTimes(1);
    });
  });
});
