import { Injectable, Logger } from '@nestjs/common';
import { StructureFormatted } from '../interfaces/structure-formatted.interface';
import { HttpService } from '@nestjs/axios';
import { Model, Types } from 'mongoose';
import { Structure, StructureDocument } from '../schemas/structure.schema';
import { InjectModel } from '@nestjs/mongoose';
import { lastValueFrom } from 'rxjs';
import * as config from './structures-export-configs/services-config.json';
import { Week } from '../../shared/schemas/week.schema';
import { Day } from '../schemas/day.schema';
import { Time } from '../schemas/time.schema';
import { DataGouvStructure } from '../interfaces/data-gouv-structure.interface';
import { StructuresSearchService } from './structures-search.service';
import { StructuresService } from './structures.service';
import { MailerService } from '../../mailer/mailer.service';
interface Condition {
  address?: { street?: string; numero?: string };
  contactMail?: string;
}
// historically, contactMail has been set to an empty string when a structure is deleted
type ContactMailCondition = string | { $in: (string | null)[] };

@Injectable()
export class StructuresImportService {
  private readonly logger = new Logger(StructuresImportService.name);
  constructor(
    private readonly httpService: HttpService,
    private structuresSearchService: StructuresSearchService,
    private structuresService: StructuresService,
    private readonly mailerService: MailerService,
    @InjectModel(Structure.name) private structureModel: Model<StructureDocument>
  ) {}

  //@Cron(CronExpression.EVERY_DAY_AT_MIDNIGHT)
  public async importDataGouvStructures(): Promise<void> {
    this.logger.log(`Importing all structures from data.gouv dataset...`);
    try {
      const dataset = await this.fetchDataset();
      const rhoneStructures = this.filterStructuresByCode(dataset, '69');
      this.logger.log(`Found ${rhoneStructures.length} structures from rhône in data.gouv dataset...`);
      await this.processStructures(rhoneStructures);
    } catch (error) {
      this.logger.error(`Error while importing data.gouv dataset: ${error}`);
    }
  }

  private async fetchDataset(): Promise<DataGouvStructure[]> {
    const response = await lastValueFrom(
      // Fichier YYYYMMDD-data-inclusion-lieux-de-mediation-numeriques-national-sans-doublons.json
      // sur https://www.data.gouv.fr/fr/datasets/lieux-de-mediation-numerique-sur-le-territoire-national-fournis-par-data-inclusion-1/#/resources
      this.httpService.get<DataGouvStructure[]>(
        'https://www.data.gouv.fr/fr/datasets/r/bdfef095-f3a9-4e66-84bb-ea05d5c9c3a5'
      )
    );
    return response.data;
  }

  private filterStructuresByCode(structures: DataGouvStructure[], code: string): DataGouvStructure[] {
    return structures.filter((struct) => struct.code_postal.startsWith(code));
  }

  private async processStructures(structures: DataGouvStructure[]): Promise<void> {
    let newCount = 0;
    let skippedCount = 0;
    let errorCount = 0;

    for (const structure of structures) {
      try {
        if (await this.doesAlreadyExist(structure)) {
          skippedCount++;
        } else {
          await this.createStructure(structure);
          newCount++;
        }
      } catch (error) {
        errorCount++;
        this.logger.error(
          `Failed to process structure with ID ${structure.id || 'unknown'}: ${error.message}`,
          error.stack
        );
      }
    }

    this.logger.log(
      `Created ${newCount} new structures, skipped ${skippedCount} structures, and encountered errors in ${errorCount} structures...`
    );

    if (newCount > 0) {
      try {
        await this.structuresService.sendAdminNotificationAfterImportingStructure(
          this.mailerService.config.templates.adminStructureImport.ejs,
          this.mailerService.config.templates.adminStructureImport.json
        );
      } catch (notificationError) {
        this.logger.error(`Failed to send admin notification: ${notificationError.message}`, notificationError.stack);
      }
    }
  }

  private async createStructure(structure: DataGouvStructure): Promise<void> {
    const formattedStructure = await this.formatToResinSchema(structure);
    const newStructure = new this.structureModel(formattedStructure);
    const createdStructure = await newStructure.save();
    // call indexStructure, updateDenormalizedFields accordingly when creating a new structure
    await this.structuresSearchService.indexStructure(createdStructure);
    await this.structuresService.updateDenormalizedFields(new Types.ObjectId(createdStructure._id));
  }

  private async doesAlreadyExist(structure: DataGouvStructure): Promise<boolean> {
    if (structure.source.toLowerCase() === 'res-in') {
      return true;
    }

    // if this structure has already been imported with the same id from data.gouv
    if ((await this.structureModel.find({ idDataGouv: structure.id }).exec()).length) {
      return true;
    }

    let addressCondition = {};
    // test if string starts with a number (22, 13bis ...)
    if (/^\d/.test(structure.adresse)) {
      const addressParts = structure.adresse.split(' ');
      const numero = addressParts.shift();
      const street = addressParts.join(' ');
      addressCondition = { 'address.street': street, 'address.numero': numero };
    } else {
      addressCondition = { 'address.street': structure.adresse };
    }

    const contactMailCondition: { contactMail: ContactMailCondition } = { contactMail: structure.courriel };

    if (!structure.courriel) {
      contactMailCondition.contactMail = { $in: [null, ''] };
    }

    const conditions: Condition[] = [addressCondition, contactMailCondition];
    const count = await this.structureModel.countDocuments({ $and: conditions }).exec();

    return count > 0;
  }

  private async formatToResinSchema(structure: DataGouvStructure): Promise<Structure> {
    const formatted = new Structure({
      idDataGouv: structure.id,
      structureName: structure.nom,
      address: this.formatAddress(structure.adresse, structure.commune, structure.code_postal, structure.code_insee),
      updatedAt: new Date(structure.date_maj),
      categories: this.servicesToCategories(
        structure.services,
        structure.conditions_acces,
        structure.publics_accueillis
      ),
      coord: [structure.longitude, structure.latitude],
      description: structure.presentation_detail ?? null,
      contactPhone: structure.telephone?.replace(/^\+33/, '0') ?? null,
      contactMail: structure.courriel ?? null,
      hours: this.convertFromOSMFormat(structure.horaires),
      accountVerified: true,
      pmrAccess: false,
      nbComputers: 0,
      nbPrinters: 0,
      nbScanners: 0,
      freeWorkShop: 'Non',
      structureType: null,
      website: structure.site_web ?? null,
      dataShareConsentDate: null,
      exceptionalClosures: null,
      facebook: null,
      instagram: null,
      twitter: null,
      personalOffers: [],
      otherDescription: null,
      linkedin: null,
    });

    return formatted;
  }

  private formatAddress(
    adresse: StructureFormatted['adresse'],
    commune: StructureFormatted['commune'],
    code_postal: StructureFormatted['code_postal'],
    code_insee: StructureFormatted['code_insee']
  ): StructureDocument['address'] {
    let numero = null;
    let street = null;

    // test if string starts with a number (22, 13bis ...)
    if (/^\d/.test(adresse)) {
      const addressParts = adresse.split(' ');
      numero = addressParts.shift();
      street = addressParts.join(' ');
    } else {
      street = adresse;
    }
    return {
      numero: numero,
      street: street,
      commune: commune,
      postcode: code_postal,
      inseeCode: code_insee,
    };
  }

  private servicesToCategories(
    services: StructureFormatted['services'],
    conditions_acces: StructureFormatted['conditions_acces'],
    publics_accueillis: StructureFormatted['publics_accueillis']
  ): StructureDocument['categories'] {
    const categories: StructureDocument['categories'] = {
      labelsQualifications: [],
      accessModality: [],
      onlineProcedures: [],
      selfServiceMaterial: [],
      equipmentsServices: [],
      baseSkills: [],
      advancedSkills: [],
      age: [],
      languageAndIlliteracy: [],
      handicaps: [],
      solidarityMaterial: [],
      ctm: [],
      genre: [],
    };
    if (!services) {
      return categories;
    }

    const servicesArray = services.split(';');

    for (const [stringToMatch, serviceConfig] of Object.entries(config)) {
      if (servicesArray.includes(stringToMatch)) {
        for (const category of serviceConfig.categories) {
          categories[category] = [...new Set([...categories[category], ...serviceConfig.values])];
        }
      }
    }

    if (conditions_acces === 'Gratuit : Je peux accéder gratuitement au lieu et à ses services') {
      categories.accessModality.push('accesLibre');
    }

    const ageMappingReverse = {
      'Familles/enfants': 'family',
      'Jeunes (16-26 ans)': 'young',
      Adultes: 'adults',
      'Seniors (+ 65 ans)': 'old',
    };

    const handicapsMappingReverse = {
      'Déficience visuelle': 'blindness',
      Surdité: 'deafness',
      'Handicaps mentaux : déficiences limitant les activités d’une personne': 'mentalDisability',
    };

    const languageMappingReverse = {
      'Personnes en situation d’illettrisme': 'illettrisme',
      'Public langues étrangères': 'foreignLanguage',
    };

    const otherMappingReverse = {
      'Uniquement femmes': 'uniquementFemmes',
    };

    if (publics_accueillis !== undefined) {
      const allPublics = publics_accueillis.split(';');
      allPublics.forEach((value) => {
        if (value in ageMappingReverse) {
          categories.age.push(ageMappingReverse[value]);
        } else if (value in handicapsMappingReverse) {
          categories.handicaps.push(handicapsMappingReverse[value]);
        } else if (value in languageMappingReverse) {
          categories.languageAndIlliteracy.push(languageMappingReverse[value]);
        } else if (value in otherMappingReverse) {
          categories.genre.push(otherMappingReverse[value]);
        }
      });
    }
    return categories;
  }

  private convertFromOSMFormat(osmHours: string): Week {
    const daysOfWeek = ['mo', 'tu', 'we', 'th', 'fr', 'sa', 'su'];
    const fullDaysOfWeek = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'];
    const week = new Week();

    // initialize all days as closed
    fullDaysOfWeek.forEach((day) => {
      week[day] = new Day();
      week[day].open = false;
      week[day].time = [];
    });

    if (!osmHours) {
      return week;
    }

    // Normalize the osmHours string by removing any spaces after semicolons and then split it into individual time entries.
    const entries = osmHours.replace(/;\s+/g, ';').split(';');
    entries.forEach((entry) => {
      const trimmedEntry = entry.trim();

      // Check for "off" conditions first
      if (trimmedEntry.includes('off')) {
        const daysToClose = trimmedEntry.replace(' off', ''); // remove 'off'

        if (daysToClose.includes('-')) {
          // Range of days
          const [startDay, endDay] = daysToClose.split('-').map((d) => d.trim());
          const startIndex = daysOfWeek.indexOf(startDay.toLowerCase());
          const endIndex = daysOfWeek.indexOf(endDay.toLowerCase());

          for (let i = startIndex; i <= endIndex; i++) {
            week[fullDaysOfWeek[i]].open = false;
            week[fullDaysOfWeek[i]].time = [];
          }
        } else {
          // Discrete days, split by comma
          const splitDays = daysToClose.split(',');
          splitDays.forEach((day) => {
            const dayIndex = daysOfWeek.indexOf(day.trim().toLowerCase());
            week[fullDaysOfWeek[dayIndex]].open = false;
            week[fullDaysOfWeek[dayIndex]].time = [];
          });
        }
      } else {
        const [days, times] = trimmedEntry.split(' ');

        // Handle day ranges like Mo-Fr or individual days
        if (days.includes('-')) {
          const [startDay, endDay] = days.split('-');
          const startIndex = daysOfWeek.indexOf(startDay.toLowerCase());
          const endIndex = daysOfWeek.indexOf(endDay.toLowerCase());

          const timeIntervals = times.split(',');
          for (let i = startIndex; i <= endIndex; i++) {
            week[fullDaysOfWeek[i]].open = true;
            timeIntervals.forEach((interval) => {
              const [opening, closing] = interval.trim().split('-');
              week[fullDaysOfWeek[i]].time.push({ opening, closing } as Time);
            });
          }
        } else {
          // Discrete days, split by comma
          const splitDays = days.split(',');
          splitDays.forEach((day) => {
            const dayIndex = daysOfWeek.indexOf(day.trim().toLowerCase());

            // Clear previous timings for the day if they exist
            week[fullDaysOfWeek[dayIndex]].time = [];

            const timeSplits = times.split(',');
            timeSplits.forEach((time) => {
              const [opening, closing] = time.split('-');
              week[fullDaysOfWeek[dayIndex]].open = true;
              week[fullDaysOfWeek[dayIndex]].time.push({ opening, closing } as Time);
            });
          });
        }
      }
    });

    return week;
  }
}
