import { HttpModule } from '@nestjs/axios';
import { HttpStatus } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import { Types } from 'mongoose';
import { UsersServiceMock } from '../../../test/mock/services/user.mock.service';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { Employer } from '../schemas/employer.schema';
import { EmployerService } from '../services/employer.service';
import { UsersService } from '../services/users.service';
import { EmployerController } from './employer.controller';

describe('EmployerController', () => {
  let employerController: EmployerController;
  let userService: UsersService;

  const employerServiceMock = {
    findAll: jest.fn(),
    searchByName: jest.fn(),
    findByName: jest.fn(),
    create: jest.fn(),
    deleteByName: jest.fn(),
    initEmployerIndex: jest.fn(),
    findOne: jest.fn(),
    deleteOneId: jest.fn(),
    findAllValidated: jest.fn(),
    findAllUnvalidated: jest.fn(),
    validate: jest.fn(),
    update: jest.fn(),
    mergeEmployer: jest.fn(),
  };

  beforeEach(async () => {
    const module = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule],
      providers: [
        {
          provide: UsersService,
          useClass: UsersServiceMock,
        },
        {
          provide: EmployerService,
          useValue: employerServiceMock,
        },
        {
          provide: getModelToken('Employer'),
          useValue: Employer,
        },
      ],
      controllers: [EmployerController],
    }).compile();

    employerController = module.get<EmployerController>(EmployerController);
    userService = module.get<UsersService>(UsersService);
  });

  it('should be defined', () => {
    expect(employerController).toBeDefined();
  });

  describe('findAll', () => {
    it('should call service.findAll', async () => {
      await employerController.findAll();
      expect(employerServiceMock.findAll).toHaveBeenCalled();
    });
    it('should service.searchByName', async () => {
      await employerController.findAll({ search: 'CA' });
      expect(employerServiceMock.searchByName).toHaveBeenCalled();
    });
  });

  describe('Create Employer', () => {
    const RequestWithRegularUser = { user: { _id: '6036721022462b001334c4bb', role: 0 } };
    it('should not create if employer already exists', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Sopra',
        validated: true,
      });
      try {
        await employerController.createEmployer({ name: 'Sopra' }, RequestWithRegularUser);
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Employer already exist');
        expect(e.status).toBe(HttpStatus.UNPROCESSABLE_ENTITY);
      }
    });
    it('should create a employer', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce(null);
      employerServiceMock.create.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Sopra',
        validated: true,
      });
      await employerController.createEmployer({ name: 'Sopra' }, RequestWithRegularUser);
      expect(employerServiceMock.create).toHaveBeenCalled();
    });
    it('should call create with send notification to true if admin', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce(null);
      const employer = {
        name: 'Sopra',
      };
      employerServiceMock.create.mockResolvedValueOnce({ ...employer, validated: true });
      const req = { user: { _id: '6036721022462b001334c4bb', role: 1 } };

      await employerController.createEmployer(employer, req);
      expect(employerServiceMock.create).toHaveBeenCalledWith(employer, true, false);
    });
  });

  describe('resetES', () => {
    it('should reset search index', async () => {
      await employerController.resetES();
      expect(employerServiceMock.initEmployerIndex).toHaveBeenCalled();
    });
  });

  describe('Validate Employer', () => {
    it('should validate an employer', async () => {
      await employerController.validateEmployer({ id: '6231aefe76598527c8d0b5bc' });
      expect(employerServiceMock.validate).toHaveBeenCalled();
    });
  });

  describe('update Employers', () => {
    it('should update an employer', async () => {
      await employerController.updateEmployer('6231aefe76598527c8d0b5bc', { name: 'SopraMod' });
      expect(employerServiceMock.update).toHaveBeenCalled();
    });
  });

  describe('Get Employers', () => {
    it('should call all validated employers and populate them with users attached to it', async () => {
      const spyer = jest.spyOn(userService, 'populateEmployersWithUsers');
      employerServiceMock.findAllValidated.mockResolvedValueOnce([
        {
          _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Metro',
          validated: true,
        },
        {
          _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Sopra',
          validated: true,
        },
      ]);
      await employerController.findValidatedEmployers();
      expect(employerServiceMock.findAllValidated.mock.calls.length).toBe(1);
      expect(spyer.mock.calls.length).toBe(1);
    });

    it('should call all unvalidated employers and populate them with users attached to it', async () => {
      const spyer = jest.spyOn(userService, 'populateEmployersWithUsers');
      employerServiceMock.findAllUnvalidated.mockResolvedValueOnce([
        {
          _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Metro',
          validated: false,
        },
        {
          _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Sopra',
          validated: false,
        },
      ]);
      await employerController.findUnvalidatedEmployers();
      expect(employerServiceMock.findAllUnvalidated.mock.calls.length).toBe(1);
      expect(spyer.mock.calls.length).toBe(1);
    });
  });
  describe('Edit Employer', () => {
    it('should update employer', async () => {
      await employerController.updateEmployer('6231aefe76598527c8d0b5bc', { name: 'SopraMod' });
      expect(employerServiceMock.update).toHaveBeenCalled();
    });

    it('should delete an unvalidated employer and replace all its occurrence with a chosen validated employer', async () => {
      await employerController.mergeEmployer({
        sourceEmployerId: '6231aefe76598527c8d0b5ba',
        targetEmployerId: '6231aefe76598527c8d0b5bc',
      });
      expect(employerServiceMock.mergeEmployer).toHaveBeenCalled();
    });
  });
  describe('Delete Employer', () => {
    it('should not be able to delete a non existing employer', async () => {
      employerServiceMock.findOne.mockResolvedValueOnce(null);
      try {
        await employerController.deleteEmployer({ id: '6231aefe76598527c8d0b5bc' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toEqual('Employer does not exists');
        expect(e.status).toEqual(HttpStatus.NOT_FOUND);
      }
    });
    it('should not delete employer if a user is linked', async () => {
      employerServiceMock.findOne.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Sopra',
        validated: true,
      });
      try {
        await employerController.deleteEmployer({ id: '6231aefe76598527c8d0b5bc' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toEqual('Cannot delete employer. It has user(s) attached to it.');
        expect(e.status).toEqual(HttpStatus.FORBIDDEN);
      }
    });
    it('should delete employer', async () => {
      employerServiceMock.findOne.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Sopra',
        validated: true,
      });
      await employerController.deleteEmployer({ id: '6231aefe76598527c8d0b5ba' });
      expect(employerServiceMock.deleteOneId).toHaveBeenCalled();
    });
  });
});
