import {
  Body,
  Controller,
  Delete,
  Get,
  HttpException,
  HttpStatus,
  Logger,
  Param,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { ApiBearerAuth, ApiOperation, ApiParam, ApiTags } from '@nestjs/swagger';
import { MergeEmployerDto } from '../../admin/dto/merge-employer.dto';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';
import { hasAdminRole } from '../../shared/utils';
import { Roles } from '../decorators/roles.decorator';
import { CreateEmployerDto } from '../dto/create-employer.dto';
import { RolesGuard } from '../guards/roles.guard';
import { IEmployer } from '../interfaces/employer.interface';
import { Employer } from '../schemas/employer.schema';
import { EmployerService } from '../services/employer.service';
import { UsersService } from '../services/users.service';

@ApiTags('employer')
@Controller('employer')
export class EmployerController {
  private readonly logger = new Logger(EmployerController.name);

  constructor(private employerService: EmployerService, private usersService: UsersService) {}

  /**
   * Find all employer. If search is given as param, filter on it. Otherwise return everything
   */
  @Get()
  public async findAll(@Query() query?: { search: string }): Promise<Employer[]> {
    if (query && query.search) {
      this.logger.debug(`findAll with query ${query.search}`);
      return this.employerService.searchByName(query.search);
    }
    this.logger.debug('findAll without query');
    return this.employerService.findAll();
  }

  /**
   * Create new employer if does not exist
   */
  @Post()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth('JWT')
  @ApiParam({ name: 'newEmployer', type: CreateEmployerDto, required: true })
  public async createEmployer(@Body() newEmployer: CreateEmployerDto, @Request() req): Promise<Employer> {
    this.logger.debug(`createEmployer: ${newEmployer.name}`);
    const existingEmployer = await this.employerService.findByName(newEmployer.name);
    if (existingEmployer) {
      this.logger.warn(`Employer already exist: ${newEmployer.name}`);
      throw new HttpException('Employer already exist', HttpStatus.UNPROCESSABLE_ENTITY);
    }
    // if user is admin, do not send notification
    if (hasAdminRole(req.user)) {
      return this.employerService.create(newEmployer, true, false);
    }
    return this.employerService.create(newEmployer);
  }

  /**
   * Init or reset search index for employer
   */
  @Post('searchIndex')
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Roles('admin')
  public async resetES(): Promise<Employer[]> {
    return await this.employerService.initEmployerIndex();
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Get('validated')
  @ApiOperation({ description: 'Get validated employers populated with users attached to it' })
  public async findValidatedEmployers() {
    return this.employerService.findAllValidated().then(async (employers: IEmployer[]) => {
      return this.usersService.populateEmployersWithUsers(employers);
    });
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Get('unvalidated')
  @ApiOperation({ description: 'Get unvalidated employers for validation or merge' })
  public async findUnvalidatedEmployers() {
    return this.employerService.findAllUnvalidated().then(async (employers: IEmployer[]) => {
      return this.usersService.populateEmployersWithUsers(employers);
    });
  }

  /**
   * All users attached to this employer will be affected to the targeted employer
   * The original unvalidated employer will be deleted
   */
  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Put('merge')
  public async mergeEmployer(@Body() mergeEmployer: MergeEmployerDto) {
    return this.employerService.mergeEmployer(mergeEmployer);
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Post('validate/:id')
  @ApiParam({ name: 'id', type: String, required: true })
  @ApiOperation({ description: 'Validate employer' })
  public async validateEmployer(@Param() params) {
    return this.employerService.validate(params.id);
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Put(':id')
  public async updateEmployer(@Param('id') id: string, @Body() body: CreateEmployerDto) {
    return this.employerService.update(id, body);
  }

  /**
   * Delete Employer if exist
   */
  @Delete(':id')
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Roles('admin')
  @ApiParam({ name: 'employer', type: CreateEmployerDto, required: true })
  public async deleteEmployer(@Param() params): Promise<Employer> {
    this.logger.debug(`deleteEmployer: ${params.id}`);
    const researchedEmployer = await this.employerService.findOne(params.id);
    // look for any relations within user collection, reject action if found
    if (researchedEmployer === null) {
      throw new HttpException('Employer does not exists', HttpStatus.NOT_FOUND);
    }
    const isEmployerLinked = await this.usersService.isEmployerLinkedToUser(researchedEmployer._id);
    if (isEmployerLinked) {
      throw new HttpException('Cannot delete employer. It has user(s) attached to it.', HttpStatus.FORBIDDEN);
    }
    return this.employerService.deleteOneId(params.id);
  }
}
