import { HttpModule } from '@nestjs/axios';
import { HttpStatus } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import { Types } from 'mongoose';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { CreateJobDto } from '../dto/create-job.dto';
import { Job } from '../schemas/job.schema';
import { User } from '../schemas/user.schema';
import { JobsService } from '../services/jobs.service';
import { UsersService } from '../services/users.service';
import { JobsController } from './jobs.controller';

describe('JobsController', () => {
  let jobsController: JobsController;

  const jobServiceMock = {
    findAll: jest.fn(),
    findByName: jest.fn(),
    create: jest.fn(),
    findOne: jest.fn(),
    deleteOneId: jest.fn(),
    findAllUnvalidated: jest.fn(),
    createJobFromAdmin: jest.fn(),
    validate: jest.fn(),
    update: jest.fn(),
    mergeJob: jest.fn(),
    deleteInvalidJob: jest.fn(),
  };

  const userServiceMock = {
    populateJobsWithUsers: jest.fn(),
    isJobLinkedToUser: jest.fn(),
  };

  beforeEach(async () => {
    const module = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule],
      providers: [
        {
          provide: JobsService,
          useValue: jobServiceMock,
        },
        {
          provide: UsersService,
          useValue: userServiceMock,
        },
        {
          provide: getModelToken('Job'),
          useValue: Job,
        },
        {
          provide: getModelToken('User'),
          useValue: User,
        },
      ],
      controllers: [JobsController],
    }).compile();

    jobsController = module.get<JobsController>(JobsController);
  });

  it('should be defined', () => {
    expect(jobsController).toBeDefined();
  });

  describe('findAll', () => {
    it('should findAll', async () => {
      await jobsController.findAll();
      expect(jobServiceMock.findAll).toHaveBeenCalled();
    });
  });

  describe('createJob', () => {
    const requestWithRegularUser = { user: { _id: '6036721022462b001334c4bb', role: 0 } };
    const newJob: CreateJobDto = {
      name: 'Dev',
      hasPersonalOffer: false,
    };
    it('should not create if job already exists', async () => {
      jobServiceMock.findByName.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Dev',
        validated: true,
      });
      try {
        await jobsController.createJob(requestWithRegularUser, newJob);
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Job already exist');
        expect(e.status).toBe(HttpStatus.UNPROCESSABLE_ENTITY);
      }
    });
    it('should create a job', async () => {
      jobServiceMock.findByName.mockResolvedValueOnce(null);
      await jobsController.createJob(requestWithRegularUser, newJob);
      expect(jobServiceMock.create).toHaveBeenCalled();
    });
    it('should call create with send notification to true if admin', async () => {
      jobServiceMock.findByName.mockResolvedValueOnce(null);
      const req = { user: { _id: '6036721022462b001334c4bb', role: 1 } };
      await jobsController.createJob(req, newJob);
      expect(jobServiceMock.create).toHaveBeenCalledWith(newJob, true, newJob.hasPersonalOffer, false);
    });
  });
  describe('validateJob', () => {
    it('should validate a given job', async () => {
      await jobsController.validateJob({ id: '6231aefe76598527c8d0b5bc' });
      expect(jobServiceMock.validate).toHaveBeenCalled();
    });
  });
  describe('findValidatedJobs', () => {
    it('should call all validated jobs and populate them with users attached to it', async () => {
      await jobsController.findValidatedJobs();
      expect(jobServiceMock.findAll).toHaveBeenCalled();
      expect(userServiceMock.populateJobsWithUsers).toHaveBeenCalled();
    });
  });
  describe('findUnvalidatedJobs', () => {
    it('should call all unvalidated jobs and populate them with users attached to it', async () => {
      await jobsController.findUnvalidatedJobs();
      expect(jobServiceMock.findAllUnvalidated).toHaveBeenCalled();
      expect(userServiceMock.populateJobsWithUsers).toHaveBeenCalled();
    });
  });

  describe('updateJob', () => {
    it('should update job', async () => {
      await jobsController.updateJob('6231aefe76598527c8d0b5bc', {
        name: 'DevMod',
        hasPersonalOffer: false,
        jobsGroup: null,
      });
      expect(jobServiceMock.update).toHaveBeenCalled();
    });
  });

  describe('mergeJob', () => {
    it('should call mergeJob', async () => {
      await jobsController.mergeJob({
        sourceJobId: '6231aefe76598527c8d0b5ba',
        targetJobId: '6231aefe76598527c8d0b5bc',
      });
      expect(jobServiceMock.mergeJob).toHaveBeenCalled();
    });
  });
  describe('deleteJob', () => {
    it('should not be able to delete a non existing job', async () => {
      jobServiceMock.findOne.mockResolvedValueOnce(null);
      try {
        await jobsController.deleteJob({ id: '6231aefe76598527c8d0b5ba' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toEqual('Job does not exists');
        expect(e.status).toEqual(HttpStatus.NOT_FOUND);
      }
    });

    it('should not delete job if a user is linked', async () => {
      jobServiceMock.findOne.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Dev',
        validated: true,
      });
      userServiceMock.isJobLinkedToUser.mockResolvedValueOnce(true);
      try {
        await jobsController.deleteJob({ id: '6231aefe76598527c8d0b5bc' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toEqual('Cannot delete job. It has user(s) attached to it.');
        expect(e.status).toEqual(HttpStatus.FORBIDDEN);
      }
    });
    it('should delete job', async () => {
      jobServiceMock.findOne.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Dev',
        validated: true,
      });
      userServiceMock.isJobLinkedToUser.mockResolvedValueOnce(false);
      await jobsController.deleteJob({ id: '6231aefe76598527c8d0b5ba' });
      expect(jobServiceMock.deleteOneId).toHaveBeenCalled();
    });
  });
});
