import { Body, Controller, Get, Logger, Post, Query, UseGuards } from '@nestjs/common';
import { ApiBearerAuth, ApiBody, ApiTags } from '@nestjs/swagger';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';
import { Roles } from '../decorators/roles.decorator';
import { RolesGuard } from '../guards/roles.guard';
import { IUserRegistry, UserRegistryPaginatedResponse } from '../interfaces/userRegistry.interface';
import { UserRegistryService } from '../services/userRegistry.service';

@ApiTags('userRegistry')
@Controller('userRegistry')
export class UsersRegistryController {
  private readonly logger = new Logger(UsersRegistryController.name);
  constructor(private userRegistryService: UserRegistryService) {}

  /**
   * Find all users in Registry. If search is given as param, filter on it. Otherwise return everything. Results comes paginated (default is 20 per page, defined in service)
   */
  @Post()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth('JWT')
  @ApiBody({ required: false })
  public async findAll(
    @Query() query?: { search: string },
    @Body() filters?: { jobsGroup?: string[]; employer?: string[]; page: number }
  ): Promise<UserRegistryPaginatedResponse> {
    this.logger.debug(`findAll with query '${query.search}' and ${JSON.stringify(filters)}`);
    return this.userRegistryService.findUsersByNameEmployerOrJobsGroup(
      query.search || '',
      filters?.page || 1,
      filters?.jobsGroup || [],
      filters?.employer || []
    );
  }

  /**
   * Return every user in registry
   */
  @Get('/count')
  public async findAllCount(): Promise<number> {
    this.logger.debug('findAll UserRegistry');
    return this.userRegistryService.countAllUserRegistry();
  }

  /**
   * Init or reset search index for users
   */
  @Post('searchIndex')
  @UseGuards(JwtAuthGuard, RolesGuard)
  @Roles('admin')
  public async resetES(): Promise<IUserRegistry[]> {
    this.logger.debug('reset ES UserRegistry');
    return await this.userRegistryService.initUserRegistryIndex();
  }
}
