import { HttpModule } from '@nestjs/axios';
import { HttpStatus } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import { Types } from 'mongoose';
import { mockUser, usersMockData } from '../../../test/mock/data/users.mock.data';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { MailerService } from '../../mailer/mailer.service';
import { SearchModule } from '../../search/search.module';
import { Structure } from '../../structures/schemas/structure.schema';
import { StructuresSearchService } from '../../structures/services/structures-search.service';
import { StructuresService } from '../../structures/services/structures.service';
import { TempUser } from '../../temp-user/temp-user.schema';
import { TempUserService } from '../../temp-user/temp-user.service';
import { EmailChangeDto } from '../dto/change-email.dto';
import { PasswordChangeDto } from '../dto/change-password.dto';
import { CreateUserDto } from '../dto/create-user.dto';
import { DescriptionDto } from '../dto/description.dto';
import { ProfileDto } from '../dto/profile.dto';
import { UpdateDetailsDto } from '../dto/update-details.dto';
import { User } from '../schemas/user.schema';
import { EmployerService } from '../services/employer.service';
import { JobsService } from '../services/jobs.service';
import { UsersService } from '../services/users.service';
import { PasswordResetApplyDto } from './../dto/reset-password-apply.dto';
import { PasswordResetDto } from './../dto/reset-password.dto';
import { UsersController } from './users.controller';
import { PasswordResetCheckDto } from '../dto/reset-password-check.dto';
import { mockStructure } from '../../../test/mock/data/structures.mock.data';
import { IPendingStructureToken } from '../interfaces/pending-structure-token.interface';

describe('UsersController', () => {
  let usersController: UsersController;

  const employerServiceMock = {
    findByName: jest.fn(),
  };

  const jobServiceMock = {
    findByName: jest.fn(),
  };

  const userServiceMock = {
    changeUserEmail: jest.fn(),
    changeUserPassword: jest.fn(),
    create: jest.fn(),
    deleteOne: jest.fn(),
    findById: jest.fn(),
    findOne: jest.fn(),
    isStructureClaimed: jest.fn(),
    sendResetPasswordEmail: jest.fn(),
    updateDescription: jest.fn(),
    updateStructureLinkedClaim: jest.fn(),
    updateUserDetails: jest.fn(),
    updateUserProfile: jest.fn(),
    resetPassword: jest.fn(),
    validateUser: jest.fn(),
    verifyAndUpdateUserEmail: jest.fn(),
    verifyUserExist: jest.fn(),
    findByPermalink: jest.fn(),
    checkPasswordResetToken: jest.fn(),
    updatePendingStructureLinked: jest.fn(),
    updateStructureLinked: jest.fn(),
    removeFromPendingStructureLinked: jest.fn(),
    sendStructureClaimApproval: jest.fn(),
  };

  const structureServiceMock = {
    deleteOne: jest.fn(),
    findOne: jest.fn(),
    getAllDataConsentPendingStructures: jest.fn(),
    sendAdminStructureNotification: jest.fn(),
    sendStructureJoinRequest: jest.fn(),
  };

  const tempUserServiceMock = {
    delete: jest.fn(),
    findOne: jest.fn(),
  };
  const mockJwtService = {
    sign: jest.fn(),
    decode: jest.fn<IPendingStructureToken, any>(),
  };

  beforeEach(async () => {
    const module = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule, SearchModule],
      providers: [
        UsersService,
        StructuresSearchService,
        MailerService,
        {
          provide: StructuresService,
          useValue: structureServiceMock,
        },
        {
          provide: UsersService,
          useValue: userServiceMock,
        },
        {
          provide: TempUserService,
          useValue: tempUserServiceMock,
        },
        {
          provide: EmployerService,
          useValue: employerServiceMock,
        },
        {
          provide: JobsService,
          useValue: jobServiceMock,
        },
        {
          provide: JwtService,
          useValue: mockJwtService,
        },
        {
          provide: getModelToken('TempUser'),
          useValue: TempUser,
        },
        {
          provide: getModelToken('Structure'),
          useValue: Structure,
        },
        {
          provide: getModelToken('User'),
          useValue: User,
        },
      ],
      controllers: [UsersController],
    }).compile();

    usersController = module.get<UsersController>(UsersController);
  });
  afterEach(() => {
    jest.clearAllMocks();
  });
  it('should be defined', () => {
    expect(usersController).toBeDefined();
  });

  describe('getProfile', () => {
    it('should return user', () => {
      const user = { _id: '36', email: 'a@a.com' };
      const result = usersController.getProfile({ user: user });
      expect(result).toEqual(user);
    });
  });

  describe('setProfile', () => {
    const setProfileData: ProfileDto = {
      employerName: 'Metro',
      jobName: 'Dev',
    };
    it('should return employer does not exist', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce(null);
      try {
        await usersController.setProfile({ user: { _id: '36', email: 'a@a.com' } }, setProfileData);
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Employer does not exist');
        expect(e.status).toBe(HttpStatus.BAD_REQUEST);
      }
    });
    it('should return job does not exist', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5a7'),
        name: 'CAF',
        validated: true,
      });
      jobServiceMock.findByName.mockResolvedValue(null);
      try {
        await usersController.setProfile({ user: { _id: '36', email: 'a@a.com' } }, setProfileData);
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Job does not exist');
        expect(e.status).toBe(HttpStatus.BAD_REQUEST);
      }
    });
    it('should update profile', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5a7'),
        name: 'CAF',
        validated: true,
      });
      jobServiceMock.findByName.mockResolvedValue({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5be'),
        name: 'Dev',
        validated: false,
        hasPersonalOffer: false,
      });
      await usersController.setProfile({ user: { _id: '36', email: 'a@a.com' } }, setProfileData);
      expect(userServiceMock.updateUserProfile).toHaveBeenCalledTimes(1);
    });
  });

  describe('update details', () => {
    it('should update details', async () => {
      const newDetails: UpdateDetailsDto = {
        name: 'Jean',
        surname: 'Jacque',
        phone: '0605040302',
      };
      await usersController.updateDetails({ user: { _id: '36', email: 'a@a.com' } }, newDetails);
      expect(userServiceMock.updateUserDetails).toHaveBeenCalled();
    });
  });

  describe('create', () => {
    it('should create user', async () => {
      const createUserDto = new CreateUserDto();

      userServiceMock.create.mockResolvedValueOnce(usersMockData[0]);
      const result = await usersController.create(createUserDto);

      expect(userServiceMock.create).toHaveBeenCalledTimes(1);
      expect(tempUserServiceMock.findOne).toHaveBeenCalledTimes(1);
      expect(tempUserServiceMock.delete).toHaveBeenCalledTimes(0);
      expect(result).toEqual(usersMockData[0]);
    });

    it('should create user with temp user', async () => {
      const createUserDto = new CreateUserDto();
      userServiceMock.create.mockResolvedValueOnce(usersMockData[0]);
      tempUserServiceMock.findOne.mockResolvedValueOnce({ email: 'test@test.com', pendingStructuresLink: [] });
      const result = await usersController.create(createUserDto);

      expect(userServiceMock.create).toHaveBeenCalledTimes(1);

      expect(tempUserServiceMock.findOne).toHaveBeenCalledTimes(1);
      expect(tempUserServiceMock.delete).toHaveBeenCalledTimes(1);
      expect(result).toEqual(usersMockData[0]);
    });
  });

  describe('validateUser', () => {
    it('should call validateUser', async () => {
      await usersController.validateUser({ id: 1 }, 'token');
      expect(userServiceMock.validateUser).toHaveBeenCalledTimes(1);
    });
  });

  describe('changePassword', () => {
    it('should call changeUserPassword', async () => {
      await usersController.changePassword({ user: { _id: '36', email: 'a@a.com' } }, new PasswordChangeDto());
      expect(userServiceMock.changeUserPassword).toHaveBeenCalledTimes(1);
    });
  });

  describe('changeEmail', () => {
    it('should call changeUserEmail', async () => {
      await usersController.changeEmail(new EmailChangeDto());
      expect(userServiceMock.changeUserEmail).toHaveBeenCalledTimes(1);
    });
  });

  describe('verifyAndUpdateEmail', () => {
    it('should call verifyAndUpdateUserEmail', async () => {
      await usersController.verifyAndUpdateEmail('token');
      expect(userServiceMock.verifyAndUpdateUserEmail).toHaveBeenCalledTimes(1);
    });
  });

  describe('resetPassword', () => {
    it('should call sendResetPasswordEmail', async () => {
      await usersController.resetPassword(new PasswordResetDto());
      expect(userServiceMock.sendResetPasswordEmail).toHaveBeenCalledTimes(1);
    });
  });

  describe('resetPasswordApply', () => {
    it('should call resetPassword', async () => {
      await usersController.resetPasswordApply(new PasswordResetApplyDto());
      expect(userServiceMock.resetPassword).toHaveBeenCalledTimes(1);
    });
  });

  describe('resetPasswordCheck', () => {
    it('should call checkPasswordResetToken', async () => {
      await usersController.resetPasswordCheck(new PasswordResetCheckDto());
      expect(userServiceMock.checkPasswordResetToken).toHaveBeenCalledTimes(1);
    });
  });

  describe('verifyUserExist', () => {
    it('should call verifyUserExist', async () => {
      await usersController.verifyUserExist({ user: { _id: '36', email: 'a@a.com' } }, { newMail: 'test@test.com' });
      expect(userServiceMock.verifyUserExist).toHaveBeenCalledTimes(1);
    });
  });

  describe('delete', () => {
    it('should not call isStructureClaimed if no structures are linked', async () => {
      const userWithoutStructure = usersMockData[2];
      userServiceMock.deleteOne.mockResolvedValueOnce(userWithoutStructure);
      await usersController.delete({ user: { _id: '36', email: 'a@a.com' } });
      expect(userServiceMock.deleteOne).toHaveBeenCalledTimes(1);
      expect(userServiceMock.isStructureClaimed).toHaveBeenCalledTimes(0);
    });

    it('should call isStructureClaimed for each structure linked', async () => {
      const structureFindOne = jest.spyOn(structureServiceMock, 'findOne');
      const userWithThreeStructures = usersMockData[3];
      userServiceMock.deleteOne.mockResolvedValueOnce(userWithThreeStructures);
      userServiceMock.isStructureClaimed.mockResolvedValue(null);
      userServiceMock.isStructureClaimed.mockResolvedValueOnce(userWithThreeStructures);
      await usersController.delete({ user: { _id: '36', email: 'a@a.com' } });
      expect(userServiceMock.deleteOne).toHaveBeenCalledTimes(1);
      expect(userServiceMock.isStructureClaimed).toHaveBeenCalledTimes(3);
      expect(structureFindOne).toHaveBeenCalledTimes(2);
    });
  });

  describe('dataConsentValidation', () => {
    it('should call getAllDataConsentPendingStructures', async () => {
      await usersController.dataConsentValidation({ user: { _id: '36', email: 'a@a.com' } });
      expect(structureServiceMock.getAllDataConsentPendingStructures).toHaveBeenCalledTimes(1);
    });
  });

  describe('get user', () => {
    it('should return user', async () => {
      userServiceMock.findById.mockResolvedValueOnce({ _id: '36', email: 'a@a.com' });
      const result = await usersController.getUser(1);
      expect(result).toEqual({ _id: '36', email: 'a@a.com' });
      expect(userServiceMock.findById).toHaveBeenCalledTimes(1);
    });

    it('should throw error if no user found', async () => {
      userServiceMock.findById.mockResolvedValueOnce(null);

      try {
        await usersController.getUser(1);
        expect(true).toBe(false);
      } catch (error) {
        expect(error.message).toBe('User does not exist');
        expect(error.status).toBe(HttpStatus.NOT_FOUND);
      }
      expect(userServiceMock.findById).toHaveBeenCalledTimes(1);
    });
  });

  describe('update description', () => {
    it('should call updateDescription', async () => {
      await usersController.updateDescription({ user: { _id: '36', email: 'a@a.com' } }, new DescriptionDto());
      expect(userServiceMock.updateDescription).toHaveBeenCalledTimes(1);
    });
  });

  describe('join', () => {
    it('should not find structure', async () => {
      structureServiceMock.findOne.mockResolvedValueOnce(null);
      try {
        await usersController.join('structureId', mockUser);
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Invalid Structure');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });

    it('should not find user', async () => {
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findOne.mockResolvedValueOnce(null);
      try {
        await usersController.join('structureId', mockUser);
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Invalid User');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });

    it('should call join', async () => {
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findOne.mockResolvedValueOnce(mockUser);
      userServiceMock.updatePendingStructureLinked.mockResolvedValueOnce([]);
      await usersController.join('structureId', mockUser);

      expect(structureServiceMock.sendStructureJoinRequest).toHaveBeenCalledTimes(1);
    });
  });

  describe('joinValidation', () => {
    it('should have wrong parameters', async () => {
      try {
        await usersController.joinValidation('', 'status');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Wrong parameters');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });

    it('should have expired token', async () => {
      mockJwtService.decode.mockReturnValue({ expiresAt: '', idStructure: '', userId: '' });
      try {
        await usersController.joinValidation('token', 'true');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Expired or invalid token');
        expect(e.status).toBe(HttpStatus.FORBIDDEN);
      }
    });
    it('should not be able to find structure', async () => {
      mockJwtService.decode.mockReturnValue({ expiresAt: '2999-12-31T00:00:00.000Z', idStructure: '', userId: '' });
      structureServiceMock.findOne.mockResolvedValueOnce(null);
      try {
        await usersController.joinValidation('token', 'true');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Invalid Structure');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
    it('should not be able to find user', async () => {
      mockJwtService.decode.mockReturnValue({ expiresAt: '2999-12-31T00:00:00.000Z', idStructure: '', userId: '' });
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findById.mockResolvedValueOnce(null);
      try {
        await usersController.joinValidation('token', 'true');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Invalid User');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });

    it('should not be linked to structure', async () => {
      mockJwtService.decode.mockReturnValue({
        expiresAt: '2999-12-31T00:00:00.000Z',
        idStructure: '61e926192ac971550065e275',
        userId: '',
      });
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findById.mockResolvedValueOnce(mockUser);
      try {
        await usersController.joinValidation('token', 'true');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('User not linked to structure');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });

    it('should accept and return structure', async () => {
      mockJwtService.decode.mockReturnValue({
        expiresAt: '2999-12-31T00:00:00.000Z',
        idStructure: '620e5236f25755550cb86dfd',
        userId: '',
      });
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findById.mockResolvedValueOnce({
        ...mockUser,
        pendingStructuresLink: [
          {
            id: new Types.ObjectId('620e5236f25755550cb86dfd'),
            token: 'token',
            structureName: 'a',
            createdAt: '2021-01-01T00:00:00.000Z',
          },
        ],
      });

      const result = await usersController.joinValidation('token', 'true');
      expect(userServiceMock.updateStructureLinked).toHaveBeenCalledTimes(1);
      expect(userServiceMock.removeFromPendingStructureLinked).toHaveBeenCalledTimes(1);
      expect(result).toEqual(mockStructure);
    });

    it('should refuse and return structure', async () => {
      mockJwtService.decode.mockReturnValue({
        expiresAt: '2999-12-31T00:00:00.000Z',
        idStructure: '620e5236f25755550cb86dfd',
        userId: '',
      });
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findById.mockResolvedValueOnce({
        ...mockUser,
        pendingStructuresLink: [
          {
            id: new Types.ObjectId('620e5236f25755550cb86dfd'),
            token: 'token',
            structureName: 'a',
            createdAt: '2021-01-01T00:00:00.000Z',
          },
        ],
      });

      const result = await usersController.joinValidation('token', 'false');
      expect(userServiceMock.updateStructureLinked).toHaveBeenCalledTimes(0);
      expect(userServiceMock.removeFromPendingStructureLinked).toHaveBeenCalledTimes(1);
      expect(result).toEqual(mockStructure);
    });
  });

  describe('joinCancel', () => {
    it('should fail to find structure', async () => {
      structureServiceMock.findOne.mockResolvedValueOnce(null);
      try {
        await usersController.joinCancel('structureId', 'token');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Invalid Structure');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });

    it('should fail to find user', async () => {
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findById.mockResolvedValueOnce(null);
      try {
        await usersController.joinCancel('structureId', 'token');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Invalid User');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });

    it('should throw an error pending state', async () => {
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findById.mockResolvedValueOnce(mockUser);
      try {
        await usersController.joinCancel('00e926192ac971550065e000', 'token');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('This structure is in pending state');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });

    it('should call removeFromPendingStructureLinked', async () => {
      structureServiceMock.findOne.mockResolvedValueOnce(mockStructure);
      userServiceMock.findById.mockResolvedValueOnce({
        ...mockUser,
        pendingStructuresLink: [
          {
            id: new Types.ObjectId('620e5236f25755550cb86dfd'),
            token: 'token',
            structureName: 'a',
            createdAt: '2021-01-01T00:00:00.000Z',
          },
        ],
      });
      userServiceMock.removeFromPendingStructureLinked.mockResolvedValueOnce([]);
      await usersController.joinCancel('620e5236f25755550cb86dfd', 'token');
      expect(userServiceMock.removeFromPendingStructureLinked).toHaveBeenCalledTimes(1);
    });
  });

  describe('permalink', () => {
    it('should get user by permalink', async () => {
      userServiceMock.findByPermalink.mockResolvedValueOnce(usersMockData[0]);
      const result = await usersController.getUserByPermalink('test');
      expect(userServiceMock.findByPermalink).toHaveBeenCalled();
      expect(result).toEqual(usersMockData[0]);
    });
    it('should throw error if no user found', async () => {
      userServiceMock.findByPermalink.mockResolvedValueOnce(null);
      try {
        await usersController.getUserByPermalink('test');
        expect(true).toBe(false);
      } catch (error) {
        expect(error.message).toBe('User does not exist');
        expect(error.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
  });
});
