import {
  IndicesCreateResponse,
  IndicesRefreshResponse,
  IndicesResponseBase,
  UpdateResponse,
} from '@elastic/elasticsearch/lib/api/types';
import { Injectable, Logger } from '@nestjs/common';
import { ElasticsearchService } from '@nestjs/elasticsearch';
import { escapeElasticsearchQuery, es_settings_homemade_french } from '../../shared/utils';
import { Employer, EmployerDocument } from '../schemas/employer.schema';

@Injectable()
export class EmployerSearchService {
  private index = 'employer';
  private readonly logger = new Logger(EmployerSearchService.name);

  constructor(private readonly elasticsearchService: ElasticsearchService) {}

  public async indexEmployer(employer: EmployerDocument): Promise<EmployerDocument> {
    this.logger.debug(`indexEmployer ${employer.name}`);
    this.elasticsearchService.index<Employer>({
      index: this.index,
      id: employer._id,
      document: {
        name: employer.name,
        validated: employer.validated,
      },
    });
    return employer;
  }

  public async createEmployerIndex(): Promise<IndicesCreateResponse> {
    this.logger.debug('createEmployerIndex');
    return this.elasticsearchService.indices.create({
      index: this.index,
      mappings: {
        dynamic_templates: [
          {
            strings: {
              match_mapping_type: 'string',
              mapping: {
                type: 'text',
                analyzer: 'homemade_french',
                search_analyzer: 'homemade_french_stopless',
              },
            },
          },
        ],
      },
      settings: es_settings_homemade_french,
    });
  }

  public async dropIndex(): Promise<IndicesResponseBase> {
    this.logger.debug('dropIndex');
    if (
      await this.elasticsearchService.indices.exists({
        index: this.index,
      })
    ) {
      return this.elasticsearchService.indices.delete({
        index: this.index,
      });
    }
  }

  public async deleteIndex(employer: EmployerDocument, id: string): Promise<EmployerDocument> {
    this.logger.debug('deleteIndex');
    this.elasticsearchService.delete({
      index: this.index,
      id: id,
    });
    return employer;
  }

  public async refreshIndexStructure(): Promise<IndicesRefreshResponse> {
    this.logger.debug('refreshIndexStructure');
    return this.elasticsearchService.indices.refresh({
      index: this.index,
    });
  }

  public async search(searchString: string): Promise<Employer[]> {
    this.logger.debug(`search ${searchString}`);
    searchString = searchString ? searchString + '*' : '*';
    const { hits } = await this.elasticsearchService.search<Employer>({
      index: this.index,
      from: 0,
      size: 5000,
      query: {
        query_string: {
          analyze_wildcard: true,
          query: escapeElasticsearchQuery(searchString),
          fields: ['name'],
          fuzziness: 'AUTO',
        },
      },
    });
    const sortedHits = hits.hits.filter(function (elem) {
      return elem._score >= hits.max_score / 1.5;
    });
    return sortedHits.map((item) => item._source);
  }

  public async update(employer: EmployerDocument, id: string): Promise<UpdateResponse<unknown>> {
    this.logger.debug('update');
    return this.elasticsearchService.update({
      index: this.index,
      id: id,
      body: {
        doc: {
          name: employer.name,
          validated: employer.validated,
        },
      },
    });
  }
}
