import { HttpModule } from '@nestjs/axios';
import { HttpStatus } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import { AxiosResponse } from 'axios';
import { Types } from 'mongoose';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { MailerService } from '../../mailer/mailer.service';
import { EmployerDocument } from '../schemas/employer.schema';
import { User } from '../schemas/user.schema';
import { EmployerSearchService } from './employer-search.service';
import { EmployerService } from './employer.service';
import { UsersService } from './users.service';

const mockMetroEmployer = {
  _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
  name: 'Metro',
  validated: true,
} as const;
const mockSopraEmployer = {
  _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
  name: 'Sopra',
  validated: true,
} as const;

describe('EmployerService', () => {
  let employerService: EmployerService;
  let mailerService: MailerService;

  const mockEmployerSearchService = {
    indexEmployer: jest.fn(),
    search: jest.fn(),
    dropIndex: jest.fn(),
    createEmployerIndex: jest.fn(),
    update: jest.fn(),
    deleteIndex: jest.fn(),
  };
  const mockEmployerModel = {
    create: jest.fn(),
    findOne: jest.fn(),
    findById: jest.fn(),
    deleteOne: jest.fn(),
    find: jest.fn(() => mockEmployerModel),
    sort: jest.fn(() => mockEmployerModel),
    exec: jest.fn(),
  };
  const mockUserService = {
    replaceEmployers: jest.fn(),
    getAdmins: jest.fn(),
  };
  beforeEach(async () => {
    const module = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule],
      providers: [
        EmployerService,
        {
          provide: EmployerSearchService,
          useValue: mockEmployerSearchService,
        },
        {
          provide: getModelToken('Employer'),
          useValue: mockEmployerModel,
        },
        {
          provide: UsersService,
          useValue: mockUserService,
        },
        {
          provide: getModelToken('User'),
          useValue: User,
        },
        MailerService,
      ],
    }).compile();
    employerService = module.get<EmployerService>(EmployerService);
    mailerService = module.get<MailerService>(MailerService);
  });

  it('should be defined', () => {
    expect(employerService).toBeDefined();
  });

  it('findAll', async () => {
    mockEmployerModel.sort.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce([mockMetroEmployer, mockSopraEmployer]);
    const reply = await employerService.findAll();
    expect(reply.length).toBe(2);
  });
  it('findOne', async () => {
    mockEmployerModel.findById.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce(mockSopraEmployer);
    const reply = await employerService.findOne('0001aefe76598527c8d0b5bc');
    expect(reply).toBeTruthy();
  });
  it('findAllValidated', async () => {
    mockEmployerModel.find.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce([mockSopraEmployer]);
    const reply = await employerService.findAllValidated();
    expect(reply.length).toBe(1);
  });

  it('finds all Unvalidated employers', async () => {
    mockEmployerModel.find.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce([{ ...mockMetroEmployer, validated: false }]);
    const reply = await employerService.findAllUnvalidated();
    expect(reply[0].validated).toBeFalsy();
  });

  it('finds all validated employers', async () => {
    mockEmployerModel.find.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce([mockMetroEmployer, mockSopraEmployer]);
    const reply = await employerService.findAllUnvalidated();
    expect(reply.length).toBe(2);
  });

  it('findByName', async () => {
    mockEmployerModel.findOne.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce(mockSopraEmployer);
    const reply = await employerService.findByName('Sopra');
    expect(reply).toBeTruthy();
  });

  describe('createEmployer', () => {
    it('create and send admin notification', async () => {
      mockEmployerModel.create.mockResolvedValue(mockSopraEmployer);
      mockEmployerModel.findOne.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce(mockSopraEmployer);
      jest.spyOn(employerService, 'sendAdminCreateNotification').mockResolvedValueOnce();
      const reply = await employerService.create({ name: 'Sopra' });
      expect(reply).toBeTruthy();
      expect(employerService.sendAdminCreateNotification).toHaveBeenCalledTimes(1);
    });

    it('should create and not send email to admins', async () => {
      mockEmployerModel.create.mockResolvedValueOnce(mockSopraEmployer);
      mockEmployerModel.findOne.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce(mockSopraEmployer);
      jest.spyOn(employerService, 'sendAdminCreateNotification').mockResolvedValueOnce();
      const reply = await employerService.create({ name: 'Sopra' }, true, false);
      expect(employerService.sendAdminCreateNotification).toHaveBeenCalledTimes(0);
      expect(reply).toBeTruthy();
    });
  });

  it('should deleteByName', async () => {
    mockEmployerModel.findOne.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce(mockSopraEmployer);
    mockEmployerModel.deleteOne.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce({
      n: 1,
      ok: 1,
      deletedCount: 1,
    });
    mockEmployerSearchService.deleteIndex.mockResolvedValueOnce({});
    const reply = await employerService.deleteByName('Sopra');
    expect(mockEmployerSearchService.deleteIndex).toHaveBeenCalled();
    expect(reply).toBeTruthy();
  });

  it('should searchByName', async () => {
    mockEmployerSearchService.search.mockResolvedValue({ name: 'Sopra' });
    const reply = await employerService.searchByName('Sopra');
    expect(reply).toBeTruthy();
  });

  it('should initEmployerIndex', async () => {
    mockEmployerSearchService.dropIndex.mockResolvedValueOnce({});
    mockEmployerSearchService.createEmployerIndex.mockResolvedValueOnce({});
    mockEmployerModel.sort.mockReturnThis();
    mockEmployerModel.exec.mockResolvedValueOnce([mockMetroEmployer, mockSopraEmployer]);
    mockEmployerSearchService.indexEmployer
      .mockResolvedValueOnce(mockMetroEmployer)
      .mockResolvedValueOnce(mockSopraEmployer);
    const reply = await employerService.initEmployerIndex();
    expect(reply).toBeTruthy();
    expect(reply.length).toBe(2);
  });

  describe('mergeEmployer', () => {
    it('should delete source employer', async () => {
      mockEmployerModel.findById.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce(mockSopraEmployer).mockResolvedValueOnce(mockMetroEmployer);
      mockUserService.replaceEmployers.mockResolvedValueOnce(null);
      jest.spyOn(employerService, 'deleteOneId').mockResolvedValueOnce(null);
      const reply = await employerService.mergeEmployer({
        sourceEmployerId: '623aed68c5d45b6fbbaa7e60',
        targetEmployerId: '623aed68c5d45b6fbbaa7e61',
      });
      expect(reply).toEqual(mockMetroEmployer);
    });
    it('should return error if delete source employer issue', async () => {
      mockEmployerModel.findById.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce(null).mockResolvedValueOnce(mockMetroEmployer);
      jest.spyOn(employerService, 'deleteOneId').mockRejectedValueOnce({});
      mockUserService.replaceEmployers.mockResolvedValueOnce(null);
      try {
        await employerService.mergeEmployer({
          sourceEmployerId: '623aed68c5d45b6fbbaa7e60',
          targetEmployerId: '623aed68c5d45b6fbbaa7e61',
        });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Cannot operate on employer.');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
  });

  describe('sendAdminCreateNotification', () => {
    it('should sendAdminCreateNotification', async () => {
      const employer = {
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Metro',
        validated: true,
      };
      const result: AxiosResponse = {
        data: {
          status: 200,
          content: {
            success: true,
            response: [7010],
          },
        },
        status: 200,
        statusText: 'OK',
        headers: {},
        config: {},
      };
      jest.spyOn(mailerService, 'send').mockResolvedValueOnce(result);
      const spyer = jest.spyOn(mailerService, 'send');
      mockUserService.getAdmins.mockResolvedValueOnce([
        {
          _id: '6231aefe76598527c8d0b5bc',
          validationToken:
            'cf1c74c22cedb6b575945098db42d2f493fb759c9142c6aff7980f252886f36ee086574ee99a06bc99119079257116c959c8ec870949cebdef2b293666dbca42',
          emailVerified: true,
          email: 'admin@admin.com',
          password: '$2a$12$vLQjJ9zAWyUwiFLeQDa6w.XzrlgPBhw.2GWrjog/yuEjIaZnQwmZu',
          role: 0,
          name: 'admin',
          surname: 'admin',
          personalOffers: [],
        },
      ]);
      await employerService.sendAdminCreateNotification(
        employer as EmployerDocument,
        'adminEmployerCreate.ejs',
        'adminEmployerCreate.json'
      );
      expect(spyer.mock.calls.length).toBe(1);
    });
  });

  describe('validate', () => {
    it('should validate employer', async () => {
      mockEmployerModel.findById.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce({
        _id: new Types.ObjectId('623aed68c5d45b6fbbaa7e60'),
        name: 'Sopra',
        validated: false,
        save: jest.fn().mockResolvedValueOnce(null),
      });
      const employer = await employerService.validate('623aed68c5d45b6fbbaa7e60');
      expect(employer.validated).toBe(true);
    });
    it('should throw exception', async () => {
      mockEmployerModel.findById.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce(null);
      try {
        await employerService.validate('623aed68c5d45b6fbbaa7e60');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Cannot validate employer. It might have been already validate');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
  });

  describe('update', () => {
    it('should update employer', async () => {
      mockEmployerModel.findById.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce({
        _id: new Types.ObjectId('623aed68c5d45b6fbbaa7e60'),
        name: 'Sopraaa',
        validated: false,
        save: jest.fn().mockResolvedValueOnce(null),
      });
      const employer = await employerService.update('623aed68c5d45b6fbbaa7e60', { name: 'Sopra' });
      expect(employer.name).toBe('Sopra');
    });
    it('should throw exception', async () => {
      mockEmployerModel.findById.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce(null);
      try {
        await employerService.update('623aed68c5d45b6fbbaa7e60', { name: 'Sopra' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Cannot edit employer. It was not found in database.');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
  });

  describe('deleteOneId', () => {
    it('should throw an error', async () => {
      mockEmployerModel.findById.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce(null);
      try {
        await employerService.deleteOneId('6231aefe76598527c8d0b5bc');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Invalid employer id');
        expect(e.status).toBe(HttpStatus.BAD_REQUEST);
      }
    });
    it('should delete', async () => {
      mockEmployerModel.findById.mockReturnThis();
      mockEmployerModel.exec.mockResolvedValueOnce({
        ...mockMetroEmployer,
        deleteOne: jest.fn(),
      });
      await employerService.deleteOneId('6231aefe76598527c8d0b5bc');
      expect(mockEmployerSearchService.deleteIndex).toHaveBeenCalled();
    });
  });
});
