import { HttpModule } from '@nestjs/axios';
import { HttpStatus } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test } from '@nestjs/testing';
import { Types } from 'mongoose';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { MailerService } from '../../mailer/mailer.service';
import { CreateJobsGroupsDto } from '../dto/create-jobsGroups.dto';
import { JobsService } from './jobs.service';
import { JobsGroupsService } from './jobsGroups.service';
import { UsersService } from './users.service';
import { JobsGroupsDocument } from '../schemas/jobsGroups.schema';

describe('JobsGroupsService', () => {
  let jobsGroupsService: JobsGroupsService;

  const mockJobsGroupsModel = {
    create: jest.fn(),
    find: jest.fn().mockReturnThis(),
    collation: jest.fn().mockReturnThis(),
    sort: jest.fn().mockReturnThis(),
    exec: jest.fn(),
    findOne: jest.fn().mockReturnThis(),
    findById: jest.fn().mockReturnThis(),
    deleteOne: jest.fn().mockReturnThis(),
  };

  const mockJobModel = {
    create: jest.fn(),
    find: jest.fn().mockReturnThis(),
    populate: jest.fn().mockReturnThis(),
    collation: jest.fn().mockReturnThis(),
    sort: jest.fn().mockReturnThis(),
    exec: jest.fn().mockReturnThis(),
    findOne: jest.fn(),
    findById: jest.fn(),
    deleteOne: jest.fn(),
  };

  const mockUserModel = {
    find: jest.fn(),
    findOne: jest.fn(),
    findById: jest.fn(),
  };
  const mockUserService = {
    replaceJobsGroups: jest.fn(),
    getAdmins: jest.fn(),
  };

  beforeEach(async () => {
    const module = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule],
      providers: [
        JobsGroupsService,
        {
          provide: getModelToken('JobsGroups'),
          useValue: mockJobsGroupsModel,
        },
        JobsService,
        {
          provide: getModelToken('Job'),
          useValue: mockJobModel,
        },
        {
          provide: UsersService,
          useValue: mockUserService,
        },
        {
          provide: getModelToken('User'),
          useValue: mockUserModel,
        },
        MailerService,
      ],
    }).compile();
    jobsGroupsService = module.get<JobsGroupsService>(JobsGroupsService);
  });

  it('should be defined', () => {
    expect(jobsGroupsService).toBeDefined();
  });

  describe('findAll', () => {
    it('should findAll', async () => {
      const result = [
        {
          _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Technique',
        },
      ];
      mockJobsGroupsModel.exec.mockResolvedValueOnce(result);
      const reply = await jobsGroupsService.findAll(false);
      expect(reply.length).toBe(1);
    });
    it('should findAll withJobs', async () => {
      const populateSpy = jest.spyOn(jobsGroupsService, 'populateJobsGroupsWithJobs');
      const result = [
        {
          _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Technique',
          toJSON: jest.fn(),
        },
      ];
      mockJobsGroupsModel.exec.mockResolvedValueOnce(result);
      const reply = await jobsGroupsService.findAll(true);
      expect(reply.length).toBe(1);
      expect(populateSpy).toHaveBeenCalled();
    });
  });

  it('findByName', async () => {
    mockJobsGroupsModel.exec.mockResolvedValueOnce({
      _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
      name: 'Technique',
    });
    const reply = await jobsGroupsService.findByName('Technique');
    expect(reply).toBeTruthy();
  });
  it('findOne', async () => {
    mockJobsGroupsModel.exec.mockResolvedValueOnce({
      _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
      name: 'Technique',
    });
    const reply = await jobsGroupsService.findOne('6231aefe76598527c8d0b5bc');
    expect(reply).toBeTruthy();
  });

  describe('createJobsGroups', () => {
    it('create', async () => {
      mockJobsGroupsModel.create.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Technique',
      });
      const createJobsGroups: CreateJobsGroupsDto = {
        name: 'Technique',
      };
      const reply = await jobsGroupsService.create(createJobsGroups);
      expect(reply).toBeTruthy();
    });
  });

  describe('update', () => {
    it('should throw error if not found', async () => {
      mockJobsGroupsModel.exec.mockResolvedValueOnce(null);
      try {
        await jobsGroupsService.update('623aed68c5d45b6fbbaa7e60', { name: 'Technique' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
    it('should update', async () => {
      const newJobsGroups = {
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Technique',
      };
      mockJobsGroupsModel.exec.mockResolvedValueOnce({
        _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Technique old',
        save: jest.fn().mockResolvedValueOnce(newJobsGroups),
      });
      const reply = await jobsGroupsService.update('623aed68c5d45b6fbbaa7e60', { name: 'Technique' });
      expect(reply.name).toBe('Technique');
    });
  });

  describe('populateJobsGroupsWithJobs', () => {
    it('should populate jobs groups with jobs', async () => {
      const populateSpy = jest.spyOn(jobsGroupsService, 'populateJobsGroupsWithJobs');
      const result = [
        {
          _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Technique',
          jobs: [
            {
              _id: new Types.ObjectId('6231aefe76598527c8d0b5bc'),
              name: 'Technique',
            },
          ],
        },
      ];
      mockJobsGroupsModel.exec.mockResolvedValueOnce(result);
      const jobsGroups = [
        {
          _id: new Types.ObjectId('65fd51d64bce0eed5ea1309f'),
          name: 'Pilotage de projet',
        },
        {
          _id: new Types.ObjectId('65fd51d64bce0eed5ea130a2'),
          name: 'Travail social',
        },
      ] as JobsGroupsDocument[];
      populateSpy.mockImplementationOnce(() => jobsGroups as any);
      await jobsGroupsService.populateJobsGroupsWithJobs(jobsGroups);
      expect(populateSpy).toHaveBeenCalled();
    });
  });
});
