import { HttpException, HttpStatus, Inject, Injectable, Logger, forwardRef } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import { Model, Types } from 'mongoose';
import { CreateJobsGroupsDto } from '../dto/create-jobsGroups.dto';
import { JobsGroups, JobsGroupsDocument } from '../schemas/jobsGroups.schema';
import { JobsService } from './jobs.service';

@Injectable()
export class JobsGroupsService {
  private readonly logger = new Logger(JobsGroupsService.name);

  constructor(
    @InjectModel(JobsGroups.name) private jobsGroupsModel: Model<JobsGroupsDocument>,
    @Inject(forwardRef(() => JobsService))
    private readonly jobsService: JobsService
  ) {}

  public async findAll(withJobs: boolean) {
    this.logger.debug('findAll jobsGroups');
    if (withJobs) {
      return this.jobsGroupsModel
        .find()
        .collation({ locale: 'fr' })
        .sort({ name: 1 })
        .exec()
        .then(async (jobsGroups: JobsGroupsDocument[]) => {
          return this.populateJobsGroupsWithJobs(jobsGroups);
        });
    } else {
      return this.jobsGroupsModel.find().collation({ locale: 'fr' }).sort({ name: 1 }).exec();
    }
  }

  public async findByName(name: string): Promise<JobsGroupsDocument> {
    this.logger.debug(`findByName jobsGroups: ${name}`);
    return this.jobsGroupsModel.findOne({ name }).exec();
  }

  public async findOne(idParam: string): Promise<JobsGroupsDocument> {
    this.logger.debug(`findOne jobsGroups: ${idParam}`);
    return this.jobsGroupsModel.findById(new Types.ObjectId(idParam)).exec();
  }

  public async populateJobsGroupsWithJobs(jobsGroups: JobsGroupsDocument[]) {
    return Promise.all(
      jobsGroups.map(async (jobsGroup) => {
        return {
          // .toJSON() needed because mongoose Document has weird behavior with data stored in internal _doc field, cf. https://stackoverflow.com/a/68973118 and https://mongoosejs.com/docs/api/document.html#Document.prototype.toJSON()
          ...jobsGroup.toJSON(),
          jobs: await this.jobsService.getJobsGroupsAttachedJobs(jobsGroup._id),
        };
      })
    );
  }

  public async create(jobsGroup: CreateJobsGroupsDto): Promise<JobsGroups> {
    this.logger.debug(`create jobsGroups: ${jobsGroup.name}`);
    return this.jobsGroupsModel.create(jobsGroup);
  }

  public async deleteOne(jobsGroup: JobsGroupsDocument): Promise<JobsGroups> {
    return jobsGroup.deleteOne();
  }

  public async update(jobsGroupId: string, newJobsGroup: CreateJobsGroupsDto): Promise<JobsGroups> {
    this.logger.debug(`edit JobsGroup: ${jobsGroupId}`);
    const jobsGroups = await this.jobsGroupsModel.findById(new Types.ObjectId(jobsGroupId)).exec();
    if (!jobsGroups) {
      throw new HttpException('Jobs group not found', HttpStatus.NOT_FOUND);
    }
    jobsGroups.name = newJobsGroup.name;
    jobsGroups.save();
    return jobsGroups;
  }
}
